#!/usr/bin/env python3
"""
EMBL-EBI Unichem
New API: https://www.ebi.ac.uk/unichem/api/docs
https://chembl.gitbook.io/unichem/webservices
"""
import sys,os,re,time,json,logging,requests,tqdm
import urllib,urllib.parse
import pandas as pd
#
API_HOST='www.ebi.ac.uk'
API_BASE_PATH='/unichem/api/v1'
API_BASE_URL='https://'+API_HOST+API_BASE_PATH
#
HEADERS = {"Accept":"application/json", "Content-Type":"application/json"}
#
##############################################################################
def ListSources(base_url=API_BASE_URL, fout=None):
  n_out=0; tags=None; df=None;
  response = requests.get(f"{base_url}/sources")
  logging.debug(response.text)
  result = response.json()
  for source in result['sources']:
    if not tags:
      tags = list(source.keys())
      for tag in tags[:]:
        if type(source[tag]) in (list, dict):
          logging.info(f"Ignoring field: {tag}")
          tags.remove(tag)
    df_this = pd.DataFrame({tag:[source[tag]] for tag in tags})
    if fout is None: df = pd.concat([df, df_this])
    else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
    n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  return df

##############################################################################
def GetFromSourceId(ids, src_id_in, src_id_out=None, skip=None, nmax=None, base_url=API_BASE_URL, fout=None):
  n_out=0; df=None;
  compound_tags=None; source_tags=None;
  query = {"sourceID":src_id_in, "type":"sourceID"}
  for i in tqdm.trange(len(ids)):
    if skip is not None and i<skip: continue
    id_this = ids[i]
    query["compound"] = id_this
    try:
      response = requests.post(f"{base_url}/compounds", data=json.dumps(query), headers=HEADERS)
    except Exception as e:
      logging.error(str(e))
      continue
    if response.status_code!=200:
      logging.debug(f"Status code: {response.status_code}")
      continue
    logging.debug(response.text)
    result = response.json()
    logging.debug(json.dumps(result, indent=2))
    for compound in result["compounds"]:
      logging.debug(json.dumps(compound, indent=2))
      if not compound_tags:
        compound_tags = list(compound.keys())
        for tag in compound_tags[:]:
          if type(compound[tag]) in (list, dict):
            logging.info(f"Ignoring field: {tag}")
            compound_tags.remove(tag)
      for source in compound["sources"]:
        logging.debug(json.dumps(source, indent=2))
        if not source_tags:
          source_tags = list(source.keys())
          for tag in source_tags[:]:
            if type(source[tag]) in (list, dict):
              logging.info(f"Ignoring field: {tag}")
              source_tags.remove(tag)
        if src_id_out is not None:
          if source["id"] != src_id_out:
            continue
        df_this = pd.DataFrame({'src_id_in':[src_id_in], 'src_compound_id_in':[id_this]})
        df_this = pd.concat([df_this, pd.DataFrame({f"compound_{tag}":[compound[tag]] for tag in compound_tags})], axis=1)
        df_this = pd.concat([df_this, pd.DataFrame({f"source_{tag}":[source[tag]] for tag in source_tags})], axis=1)
        if fout is None: df = pd.concat([df, df_this])
        else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
        n_out += df_this.shape[0]
    if nmax is not None:
      if i>=(nmax if skip is None else nmax+skip): break
  logging.info(f"n_out: {n_out}")
  return df

#############################################################################
def GetFromInchi(ids, inchi_rep, search_components=False, src_id_in=None, src_id_out=None, skip=None, nmax=None, base_url=API_BASE_URL, fout=None):
  n_out=0; df=None;
  compound_tags=None; source_tags=None;
  query = {"searchComponents":search_components, "type":inchi_rep}
  if src_id_in is not None: query["sourceID"] = src_id_in
  for i in tqdm.trange(len(ids)):
    if skip is not None and i<skip: continue
    id_this = ids[i]
    query["compound"] = id_this
    response = requests.post(f"{base_url}/connectivity", data=json.dumps(query), headers=HEADERS)
    if response.status_code!=200:
      logging.debug(f"Status code: {response.status_code}")
      continue
    logging.debug(response.text)
    result = response.json()
    logging.debug(json.dumps(result, indent=2))
    compound = result["searchedCompound"]
    if not compound_tags:
      compound_tags = list(compound.keys())
      for tag in compound_tags[:]:
        if type(compound[tag]) in (list, dict):
          logging.info(f"Ignoring field: {tag}")
          compound_tags.remove(tag)
    for source in result["sources"]:
      logging.debug(json.dumps(source, indent=2))
      if not source_tags:
        source_tags = list(source.keys())
        for tag in source_tags[:]:
          if type(source[tag]) in (list, dict):
            logging.info(f"Ignoring field: {tag}")
            source_tags.remove(tag)
      if src_id_out is not None:
        if source["id"] != src_id_out:
          continue
      df_this = pd.DataFrame({f"searchedCompound_{tag}":[compound[tag]] for tag in compound_tags})
      df_this = pd.concat([df_this, pd.DataFrame({f"source_{tag}":[source[tag]] for tag in source_tags})], axis=1)
      if fout is None: df = pd.concat([df, df_this])
      else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      n_out += df_this.shape[0]
    if nmax is not None:
      if i>=(nmax if skip is None else nmax+skip): break
  logging.info(f"n_out: {n_out}")
  return df




#!/usr/bin/env python3
"""
ClinicalTrials.gov
https://clinicaltrials.gov/api/gui/ref/api_urls
"""
import sys,os,re,time,json,logging,requests,tqdm
import urllib,urllib.parse
import pandas as pd
#
API_HOST='clinicaltrials.gov'
API_BASE_PATH='/api'
API_BASE_URL='https://'+API_HOST+API_BASE_PATH
#
##############################################################################
def ListStudyFields(base_url=API_BASE_URL, fout=None):
  response = requests.get(f"{base_url}/info/study_fields_list?fmt=JSON")
  logging.debug(response.text)
  result = response.json()
  df = pd.DataFrame({'StudyFields': result['StudyFields']['Fields']})
  if fout is not None: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

##############################################################################
def DataVersion(base_url=API_BASE_URL, fout=None):
  response = requests.get(f"{base_url}/info/data_vrs?fmt=JSON")
  result = response.json()
  val = result['DataVrs']
  fout.write(f"{val}\n")
  return val

##############################################################################
def ApiVersion(base_url=API_BASE_URL, fout=None):
  response = requests.get(f"{base_url}/info/api_vrs?fmt=JSON")
  result = response.json()
  val = result['APIVrs']
  fout.write(f"{val}\n")
  return val

##############################################################################
def SearchStudies(query, min_rnk, max_rnk, base_url=API_BASE_URL, fout=None):
  study_tags=None; n_out=0; df=None;
  url = f"{base_url}/query/full_studies?expr={urllib.parse.quote(query)}&fmt=JSON"
  url += f"&min_rnk={min_rnk}&max_rnk={max_rnk}"
  response = requests.get(url)
  result = response.json()
  meta = result["FullStudiesResponse"]
  meta_tags = [tag for tag in list(meta.keys()) if type(meta[tag]) not in (list,dict)]
  df_meta = pd.DataFrame({tag:[meta[tag]] for tag in meta_tags})
  df_meta.transpose().to_csv(sys.stderr, "\t", index=True, header=False)
  studies = result["FullStudiesResponse"]["FullStudies"]
  for study in studies:
    logging.debug(json.dumps(study, indent=2))
    if not study_tags:
      study_tags = list(study.keys())
      for tag in study_tags[:]:
        if type(study[tag]) in (list, dict):
          logging.info(f"Ignoring field: {tag}")
          study_tags.remove(tag)

    # Need to flatten and include these modules:
    protocol = study["Study"]["ProtocolSection"]
    identification_module = protocol["IdentificationModule"]
    description_module = protocol["DescriptionModule"]
    conditions_module = protocol["ConditionsModule"]
    design_module = protocol["DesignModule"]
    status_module = protocol["StatusModule"]
    arms_interventions_module = protocol["ArmsInterventionsModule"]
    outcomes_module = protocol["OutcomesModule"]
    eligibility_module = protocol["EligibilityModule"]
    references_module = protocol["ReferencesModule"] if "ReferencesModule" in protocol else None
    derived = study["Study"]["DerivedSection"]
    condition_browse_module = derived["ConditionBrowseModule"]
    intervention_browse_module = derived["InterventionBrowseModule"] if "InterventionBrowseModule" in derived else None

    nct_id = identification_module["NCTId"]
    brief_title = identification_module["BriefTitle"]
    official_title = identification_module["OfficialTitle"]

    df_this = pd.DataFrame({tag:[study[tag]] for tag in study_tags})
    df_this = pd.concat([df_this, pd.DataFrame({"NCTId":[nct_id], "BriefTitle":[brief_title], "OfficialTitle":[official_title]})], axis=1)
    if fout is None: df = pd.concat([df, df_this])
    else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
    n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  return df

##############################################################################



#!/usr/bin/env python3
"""
Utility functions for BioGRID REST API.
See: http://wiki.thebiogrid.org/doku.php/biogridrest
"""
###
import sys,os,re,json,time,logging,requests,tqdm
import pandas as pd
#
API_HOST="webservice.thebiogrid.org"
API_BASE_PATH=""
API_BASE_URL=f"https://{API_HOST}{API_BASE_PATH}"
#
##############################################################################
def ListOrganisms(params, base_url=API_BASE_URL, fout=None):
  url = f"{base_url}/organisms/?accesskey={params['API_KEY']}&format=json"
  response = requests.get(url)
  organisms = response.json()
  logging.debug(json.dumps(organisms, indent=2))
  df = pd.DataFrame.from_dict(organisms, orient="index")
  df.reset_index(drop=False, inplace=True)
  df.columns = ["organism_id", "organism"]
  if fout is not None: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

##############################################################################
def ListIdTypes(params, base_url=API_BASE_URL, fout=None):
  url = f"{base_url}/identifiers/?accesskey={params['API_KEY']}&format=json"
  response = requests.get(url)
  idtypes = response.json()
  logging.debug(json.dumps(idtypes, indent=2))
  df = pd.DataFrame.from_dict(idtypes, orient="index")
  df.reset_index(drop=False, inplace=True)
  if fout is not None: df.to_csv(fout, "\t", index=False, header=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

##############################################################################
def GetInteractions(params, ids, base_url=API_BASE_URL, fout=None):
  n_out=0; tags=None; df=None;
  for id_this in ids:
    url = f"{base_url}/interactions/{id_this}?accesskey={params['API_KEY']}&format=json"
    response = requests.get(url)
    result = response.json()
    logging.debug(json.dumps(result, indent=2, sort_keys=False)+'\n')
    intr = result[id_this]
    if not tags:
      tags = list(intr.keys())
      for tag in tags[:]:
        if type(intr[tag]) in (list, dict):
          logging.info(f"Ignoring field: {tag}")
          tags.remove(tag)
    df_this = pd.DataFrame({tag:[intr[tag]] for tag in tags})
    if fout is None: df = pd.concat([df, df_this])
    else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
    n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  return df

##############################################################################
def SearchInteractions(params, ids, search_params, base_url=API_BASE_URL, fout=None):
  n_out=0; tags=None; df=None;
  skip=0; chunk=1000;
  url = f"{base_url}/interactions/{id_this}?accesskey={params['API_KEY']}&format=json"
  if ids:
    url+=f"&geneList={'|'.join(ids)}"
  url+=f"&interSpeciesExcluded={str(not search_params['inc_interspecies']).lower()}"
  url+=f"&selfInteractionsExcluded={str(not search_params['inc_self']).lower()}"
  if search_params['elist']:
    url+=f"&includeEvidence={str(search_params['inc_evidence']).lower()}"
    url+=f"&evidenceList={'|'.join(search_params['elist'])}"
  if search_params['addl_idtypes']:
    url+=f"&additionalIdentifierTypes={'|'.join(search_params['addl_idtypes'])}"
  if search_params['human']:
    url+=f"&taxId=9606"
  while True:
    url_this = url+f"&start={skip}&max={chunk}"
    response = requests.get(url_this)
    if response.status_code!=200:
      logging.debug(f"Status code: {response.status_code}")
      break
    result = response.json()
    logging.debug(json.dumps(result, indent=2, sort_keys=False)+'\n')
    intrs = result
    for id_this, intr in intrs.items():
      if not tags:
        tags = list(intr.keys())
        for tag in tags[:]:
          if type(intr[tag]) in (list, dict):
            logging.info(f"Ignoring field: {tag}")
            tags.remove(tag)
      df_this = pd.DataFrame({tag:[intr[tag]] for tag in tags})
      if fout is None: df = pd.concat([df, df_this])
      else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      n_out += df_this.shape[0]
    skip+=chunk
  logging.info(f"n_out: {n_out}")
  return df

##############################################################################



#!/usr/bin/env python3
"""
https://py2neo.org/
https://py2neo.org/v4/database.html
"""
import sys,os,json,logging
import pandas as pd
import py2neo

DBHOST="localhost"
DBPORT=7687
DBSCHEME="bolt"
DBUSR="neo4j"
DBPW="neo4j"

#############################################################################
def DbConnect(dbhost=DBHOST, dbport=DBPORT, dbscheme=DBSCHEME, dbusr=DBUSR, dbpw=DBPW, secure=False):
  db=None;
  try:
    db = py2neo.GraphService(host=dbhost, port=dbport, scheme=dbscheme, secure=secure, user=dbusr, password=dbpw)
  except Exception as e:
    logging.error(f"{e}")
  return db

#############################################################################
def DbInfo(db, fout):
  logging.debug(f"db.config: {db.config}")
  df = pd.DataFrame({"uri": [db.uri],
	"kernel_version": [db.kernel_version],
	"default_graph": [db.default_graph],
	"product": [db.product]})
  df.transpose().to_csv(fout, sep="\t")

#############################################################################
def DbSummary(db, fout):
  g = db.default_graph
  df = pd.DataFrame({"nodes": [len(g.nodes)],
	"relationships": [len(g.relationships)],
	"schema": [g.schema],
	})
  df.transpose().to_csv(fout, sep="\t")

#############################################################################
def DbQuery(db, cql, fmt, fout):
  n_out=0;
  g = db.default_graph
  result = g.run(cql)
  if fmt.upper()=='JSON':
    rows = result.data()
    n_out = len(rows)
    fout.write(json.dumps(rows, indent=2)+'\n')
  else: #TSV
    df = result.to_data_frame()
    n_out = df.shape[0]
    df.to_csv(fout, '\t', index=False)
  logging.info(f"rows: {n_out}")

#############################################################################



#!/usr/bin/env python3
#############################################################################
### https://maayanlab.cloud/sigcom-lincs/#/API 
#############################################################################
import sys,os,re,argparse,json,logging
#
from ...lincs import sigcom as sigcom_lincs
#
#############################################################################
if __name__=="__main__":
  parser = argparse.ArgumentParser(description='CLUE.IO REST API client utility')
  ops = ['getResources', ]
  parser.add_argument("op", choices=ops, help='OPERATION')
  parser.add_argument("--ids", help="IDs, comma-separated")
  parser.add_argument("--i", dest="ifile", help="input file, IDs")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--nmax", type=int, default=1000, help="max results")
  parser.add_argument("--skip", type=int, default=0, help="skip results")
  parser.add_argument("--api_host", default=sigcom_lincs.Utils.API_HOST)
  parser.add_argument("--api_base_path", default=sigcom_lincs.Utils.API_BASE_PATH)
  parser.add_argument("-v", "--verbose", dest="verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  base_url = 'https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  if args.ifile:
    fin = open(args.ifile)
    ids=[];
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    fin.close()
    logging.info(f"input queries: {len(ids)}")
  elif args.ids:
    ids = re.split('[, ]+', args.ids.strip())

  if args.op=='getResources':
    if not ids: parser.error('--ids or --i required.')
    sigcom_lincs.Utils.GetResources(ids, base_url, fout)

  else:
    parser.error(f"Unsupported operation: {args.op}")



#!/usr/bin/env python3
"""
ClinicalTrials.gov
https://clinicaltrials.gov/api/gui/ref/api_urls
"""
### 
import sys,os,re,json,argparse,time,logging

from .. import clinicaltrials
#
#############################################################################
if __name__=='__main__':
  epilog='''\
'''
  parser = argparse.ArgumentParser(description='ClinicalTrials.gov API client', epilog=epilog)
  ops = ['listStudyFields',
	'showDataVersion',
	'showApiVersion',
	'searchStudies',
	]
  parser.add_argument("op", choices=ops, help='OPERATION')
  parser.add_argument("--i", dest="ifile", help="Input IDs")
  parser.add_argument("--o", dest="ofile", help="Output (TSV)")
  parser.add_argument("--ids", help="Input IDs (comma-separated)")
  parser.add_argument("--query", help="Search query")
  parser.add_argument("--api_host", default=clinicaltrials.API_HOST)
  parser.add_argument("--api_base_path", default=clinicaltrials.API_BASE_PATH)
  parser.add_argument("--min_rnk", type=int, default=1, help="Minimum rank for search results.")
  parser.add_argument("--max_rnk", type=int, default=12, help="Maximum rank for search results.")
  parser.add_argument("--skip", type=int, help="")
  parser.add_argument("--nmax", type=int, help="")
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  api_base_url = 'https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  t0=time.time()

  ids=[]
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    fin.close()
    logging.info(f"Input IDs: {len(ids)}")
  elif args.ids:
    ids = re.split(r'[,\s]+', args.ids)

  if args.op == "showApiVersion":
    clinicaltrials.ApiVersion(api_base_url, fout)

  elif args.op == "showDataVersion":
    clinicaltrials.DataVersion(api_base_url, fout)

  elif args.op == "listStudyFields":
    clinicaltrials.ListStudyFields(api_base_url, fout)

  elif args.op == "searchStudies":
    clinicaltrials.SearchStudies(args.query, args.min_rnk, args.max_rnk, api_base_url, fout)

  else:
    parser.error(f"Invalid operation: {args.op}")

  logging.info(f"Elapsed time: {time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0))}")



#!/usr/bin/env python3
"""
Neo4j client (via py2neo API)
https://neo4j.com/docs/cypher-manual
https://py2neo.org
"""
#############################################################################
import sys,os,argparse,logging
import py2neo

from .. import neo4j as util_neo4j

#############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description="Neo4j client (via py2neo API)", epilog="See https://neo4j.com/docs/cypher-manual, https://py2neo.org.")
  ops = [ 'dbinfo', 'query', 'dbsummary' ]
  parser.add_argument("op", choices=ops, help='OPERATION')
  parser.add_argument("--i", dest="ifile", help="input query file (CQL aka Cypher)")
  parser.add_argument("--cql", help="input query (CQL aka Cypher)")
  parser.add_argument("--o", dest="ofile", help="output (TSV|JSON)")
  parser.add_argument("--ofmt", choices=('TSV', 'JSON'), default='TSV')
  parser.add_argument("--dbhost", default=util_neo4j.DBHOST)
  parser.add_argument("--dbport", type=int, default=util_neo4j.DBPORT)
  parser.add_argument("--dbscheme", default=util_neo4j.DBSCHEME)
  parser.add_argument("--dbusr", default=util_neo4j.DBUSR)
  parser.add_argument("--dbpw", default=util_neo4j.DBPW)
  parser.add_argument("--secure", action="store_true", help="secure connection (TLS)")
  parser.add_argument("-v", "--verbose", dest="verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>0 else logging.ERROR))

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  if args.op == 'dbinfo':
    db = util_neo4j.DbConnect(dbhost=args.dbhost, dbport=args.dbport, dbscheme=args.dbscheme, dbusr=args.dbusr, dbpw=args.dbpw, secure=args.secure)
    util_neo4j.DbInfo(db, fout)

  elif args.op == 'dbsummary':
    db = util_neo4j.DbConnect(dbhost=args.dbhost, dbport=args.dbport, dbscheme=args.dbscheme, dbusr=args.dbusr, dbpw=args.dbpw, secure=args.secure)
    util_neo4j.DbSummary(db, fout)

  elif args.op == 'query':
    db = util_neo4j.DbConnect(dbhost=args.dbhost, dbport=args.dbport, dbscheme=args.dbscheme, dbusr=args.dbusr, dbpw=args.dbpw, secure=args.secure)
    if args.ifile:
      fin = open(args.ifile)
      cql = fin.read()
    elif args.cql:
      cql = args.cql
    else:
      parser.error('--cql or --i required for query.')
    util_neo4j.DbQuery(db, cql, args.ofmt, fout)

  else:
    parser.error(f"Unsupported operation: {args.op}")
